#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <stdint.h>
#include <string.h>
#include <error.h>
#include <sys/fsuid.h>
#include <sys/mman.h>
#include <stdlib.h>

#include "utils.h"
#include "extacl.h"

int main(int argc, char **argv) {
    int uid = atoi(argv[1]);
    int f;
    char buf[8192];
    char buf2[4096];
    char *map = NULL;
    struct extacl extacl[] = {
        {
            .type = EXTACL_USER_WRITE,
            .uid_gid = uid,
            .range_start = 0,
            .range_len = 128,
        },
        {
            .type = EXTACL_USER_WRITE,
            .uid_gid = uid,
            .range_start = 1024,
            .range_len = 8192-1024,
        },
        {
            .type = EXTACL_USER_READ,
            .uid_gid = uid,
            .range_start = 4096,
            .range_len = 4096,
        },
    };

    if (argc < 2)
        error(1, 0, "argc");

    f = open("testfile", O_CREAT | O_RDWR, 0600);
    if (f < 0)
        error(1, errno, "open");
    memset(buf, 42, sizeof(buf));
    if (write(f, buf, sizeof(buf)) != sizeof(buf))
        error(1, errno, "write");
    close(f);

    if (extacl_set_file("testfile", extacl, ARRAY_SIZE(extacl)) != ARRAY_SIZE(extacl))
        error(1, errno, "extacl_set_file");

    if (setfsuid(uid) == -1)
        error(1, errno, "setfsuid");

    f = open("testfile", O_RDWR);
    map = mmap(NULL, 4096, PROT_READ | PROT_WRITE, MAP_SHARED, f, 4096);
    if (map == MAP_FAILED)
        error(1, errno, "mmap");
    memset(map, 21, 4096);
    if (munmap(map, 4096) == -1)
        error(1, errno, "munmap private");
    close(f);

    if (setfsuid(0) == -1)
        error(1, errno, "setfsuid to root");

    f = open("testfile", O_RDONLY);
    if (f < 0)
        error(1, errno, "open read");
    if (lseek(f, 4096, SEEK_SET) != 4096)
        error(1, errno, "seek");
    if (read(f, buf2, 4096) != 4096)
        error(1, errno, "read after map");
    memset(buf, 21, 4096);
    if (memcmp(buf, buf2, 4096))
        error(1, 0, "MAP_SHARED not persisted");
    close(f);
    return 0;
}
