#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <stdint.h>
#include <string.h>
#include <error.h>
#include <sys/fsuid.h>
#include <stdlib.h>

#include "utils.h"
#include "extacl.h"

int main(int argc, char **argv) {
    int uid = atoi(argv[1]);
    int f;
    int ret;
    char buf[8192];
    char buf2[sizeof(buf)];
    struct extacl extacl[] = {
        {
            .type = EXTACL_USER_READ,
            .uid_gid = uid,
            .range_start = 0,
            .range_len = 128,
        },
    };
    struct extacl extacl2[] = {
        {
            .type = EXTACL_USER_READ,
            .uid_gid = uid,
            .range_start = 0,
            .range_len = 256,
        },
    };

    if (argc < 2)
        error(1, 0, "argc");

    f = open("testfile", O_CREAT | O_RDWR, 0600);
    if (f < 0)
        error(1, errno, "open");
    memset(buf, 42, sizeof(buf));
    if (write(f, buf, sizeof(buf)) != sizeof(buf))
        error(1, errno, "write");
    close(f);

    if (extacl_set_file("testfile", extacl, ARRAY_SIZE(extacl)) != ARRAY_SIZE(extacl))
        error(1, errno, "extacl_set_file");

    if (setfsuid(uid) == -1)
        error(1, errno, "setfsuid");

    f = open("testfile", O_RDONLY);
    if (f < 0)
        error(1, errno, "open read");

    if (setfsuid(0) == -1)
        error(1, errno, "setfsuid");

    if (extacl_set_file("testfile", extacl2, ARRAY_SIZE(extacl2)) != ARRAY_SIZE(extacl2))
        error(1, errno, "extacl_set_file");

    if (setfsuid(uid) == -1)
        error(1, errno, "setfsuid");

    // 3 possible values:
    //  - 128 - ok
    //  - 256 - extacl not cached at open time
    //  - 512 - extacl not checked at all (uid 0 cached?)
    if ((ret=read(f, buf2, 512)) != 128)
        error(1, errno, "read expected 128 got %d", ret);

    if (memcmp(buf, buf2, 128))
        error(1, 0, "memcmp");
    close(f);
    return 0;
}
